@@ -51,7 +51,7 @@
 #' @param digits digits,
 #' @param width the maximum length of the lines in the print output,
 #' @param type one of `"defactored"` or `"standard"`,
-#' @param vcov a variance–covariance matrix furnished by the user or a function to calculate one,
+#' @param vcov a variance-covariance matrix furnished by the user or a function to calculate one,
 #' @param \dots further arguments.
 #' @return An object of class `c("pcce", "panelmodel")` containing:
 #'     \item{coefficients}{the vector of coefficients,}
@@ -62,11 +62,11 @@
 #'     \item{vcov}{the covariance matrix of the coefficients,}
 #'     \item{df.residual}{degrees of freedom of the residuals,}
 #'     \item{model}{a data.frame containing the variables used for the
-#'     estimation,} \item{call}{the call,} \item{sigma}{always `NULL`,
-#'     `sigma` is here only for compatibility reasons (to allow using
-#'     the same `summary` and `print` methods as `pggls`),}
+#'     estimation,}
+#'     \item{call}{the call,}
 #'     \item{indcoef}{the matrix of individual coefficients from
-#'     separate time series regressions.}
+#'     separate time series regressions,}
+#'     \item{r.squared}{numeric, the R squared.}
 #' @export
 #' @importFrom MASS ginv
 #' @author Giovanni Millo
@@ -94,61 +94,59 @@ pcce <- function (formula, data, subset, na.action,
                    #residuals = c("defactored", "standard"),
                   index = NULL, trend = FALSE, ...) {
   
-    ## Create a Formula object if necessary (from plm.R)
-#    if (!inherits(formula, "pFormula")) formula <- pFormula(formula)
-    if (!inherits(formula, "Formula")) formula <- as.Formula(formula)
+  ## Create a Formula object if necessary (from plm.R)
+#  if (!inherits(formula, "pFormula")) formula <- pFormula(formula)
+  if (!inherits(formula, "Formula")) formula <- as.Formula(formula)
 
-    ## same as pggls but for effect, fixed at "individual" for compatibility
-    ## ind for id, tind for time, k for K, coefnam for coef.names
-    effect <- "individual"
+  ## same as pggls but for effect, fixed at "individual" for compatibility
+  ## ind for id, tind for time, k for K, coefnam for coef.names
+  effect <- "individual"
 
-    ## record call etc.
-    model <- match.arg(model)
-    model.name <- paste("cce", model, sep="")
-    data.name <- paste(deparse(substitute(data)))
-    cl <- match.call()
-    plm.model <- match.call(expand.dots = FALSE)
-    m <- match(c("formula", "data", "subset", "na.action", "effect",
-        "model", "index"), names(plm.model), 0)
-    plm.model <- plm.model[c(1, m)]
-    plm.model[[1]] <- as.name("plm")
-    ## change the 'model' in call
-    plm.model$model <- "pooling"
-    ## evaluates the call, modified with model = "pooling", inside the
-    ## parent frame resulting in the pooling model on formula, data
-    plm.model <- eval(plm.model, parent.frame())
-    index <- attr(model.frame(plm.model), "index")
-    ## group index
-    ind <- index[[1]]
-    ## time index
-    tind <- index[[2]]
-    ## set dimension variables
-    pdim <- pdim(plm.model)
-    balanced <- pdim$balanced
-    nt <- pdim$Tint$nt
-    Ti <- pdim$Tint$Ti
-    T. <- pdim$nT$T
-    n <- pdim$nT$n
-    N <- pdim$nT$N
-    ## set index names
-    time.names <- pdim$panel.names$time.names
-    id.names <- pdim$panel.names$id.names
-    coef.names <- names(coef(plm.model))
-    ## number of coefficients
-    k <- length(coef.names)
+  ## record call etc.
+  model <- match.arg(model)
+  model.name <- paste("cce", model, sep="")
+  data.name <- paste(deparse(substitute(data)))
+  cl <- match.call()
+  plm.model <- match.call(expand.dots = FALSE)
+  m <- match(c("formula", "data", "subset", "na.action", "effect",
+      "model", "index"), names(plm.model), 0)
+  plm.model <- plm.model[c(1L, m)]
+  plm.model[[1L]] <- as.name("plm")
+  ## change the 'model' in call
+  plm.model$model <- "pooling"
+  ## evaluates the call, modified with model = "pooling", inside the
+  ## parent frame resulting in the pooling model on formula, data
+  plm.model <- eval(plm.model, parent.frame())
+  index <- unclass(attr(model.frame(plm.model), "index")) # unclass for speed
+  ind  <- index[[1L]] ## individual index
+  tind <- index[[2L]] ## time index
+  ## set dimension variables
+  pdim <- pdim(plm.model)
+  balanced <- pdim$balanced
+  nt <- pdim$Tint$nt
+  Ti <- pdim$Tint$Ti
+  T. <- pdim$nT$T
+  n <- pdim$nT$n
+  N <- pdim$nT$N
+  ## set index names
+  time.names <- pdim$panel.names$time.names
+  id.names <- pdim$panel.names$id.names
+  coef.names <- names(coef(plm.model))
+  ## number of coefficients
+  k <- length(coef.names)
 
-    ## model data
-    X <- model.matrix(plm.model)
-    y <- model.response(model.frame(plm.model))
+  ## model data
+  X <- model.matrix(plm.model)
+  y <- model.response(model.frame(plm.model))
 
   ## det. *minimum* group numerosity
-  t <- min(tapply(X[ , 1], ind, length))
+  t <- min(Ti) # ==  min(tapply(X[ , 1], ind, length))
 
   ## check min. t numerosity
-  ## NB it is also possible to allow estimation if there *is* one group
-  ## with t large enough and average on coefficients removing NAs
-  ## Here we choose the explicit way: let estimation fail if we lose df
-  ## but a warning would do...
+    ## NB it is also possible to allow estimation if there *is* one group
+    ## with t large enough and average on coefficients removing NAs
+    ## Here we choose the explicit way: let estimation fail if we lose df
+    ## but a warning would do...
   if(t < (k+1)) stop("Insufficient number of time periods")
 
   ## one regression for each group i in 1..n
@@ -157,13 +155,13 @@ pcce <- function (formula, data, subset, na.action,
   ## as min(t) > k+1)
 
   ## subtract intercept from parms number and names
-    if(attr(terms(plm.model), "intercept")) {
-        k <- k-1
-        coef.names <- coef.names[-1]
-    }
+  if(attr(terms(plm.model), "intercept")) {
+      k <- k-1
+      coef.names <- coef.names[-1L]
+  }
 
   ## "pre-allocate" coefficients matrix for the n models
-  tcoef <- matrix(NA, nrow = k, ncol = n)
+  tcoef <- matrix(NA_real_, nrow = k, ncol = n)
 
   ## pre-allocate residuals lists for individual regressions
   ## (lists allow for unbalanced panels)
@@ -172,27 +170,26 @@ pcce <- function (formula, data, subset, na.action,
 
   ## CCE by-group estimation
 
-    ## must put the intercept into the group-invariant part!!
-    ## so first drop it from X
-    if(attr(terms(plm.model), "intercept")) {
-        X <- X[ , -1, drop = FALSE]
-    }
+  ## must put the intercept into the group-invariant part!!
+  ## so first drop it from X
+  if(attr(terms(plm.model), "intercept")) {
+      X <- X[ , -1L, drop = FALSE]
+  }
 
-    ## group-invariant part, goes in Hhat
-      ## between-periods transformation (take means over groups for each t)
-      be <- function(x, index, na.rm = TRUE) tapply(x, index, mean, na.rm = na.rm)
-      Xm <- apply(X, 2, FUN = be, index = tind)[tind, , drop = FALSE]
-      ym <- apply(as.matrix(as.numeric(y)), 2, FUN = be, index = tind)[tind]
+  ## group-invariant part, goes in Hhat
+    ## between-periods transformation (take means over groups for each t)
+      Xm <- Between(X, effect = tind, na.rm = TRUE)
+      ym <- as.numeric(Between(y, effect = "time", na.rm = TRUE))
 
       if(attr(terms(plm.model), "intercept")) {
-              Hhat <- cbind(ym, Xm, 1)
-          } else {
-              Hhat <- cbind(ym, Xm)
-          }
+        Hhat <- cbind(ym, Xm, 1L)
+        } else {
+          Hhat <- cbind(ym, Xm)
+      }
 
       ## prepare XMX, XMy arrays
-      XMX <- array(dim = c(k, k, n))
-      XMy <- array(dim = c(k, 1, n))
+      XMX <- array(data = NA_real_, dim = c(k, k, n))
+      XMy <- array(data = NA_real_, dim = c(k, 1L, n))
 
       ## hence calc. beta_i anyway because of vcov
 
@@ -205,13 +202,15 @@ pcce <- function (formula, data, subset, na.action,
           tHhat <- Hhat[ind == unind[i], , drop = FALSE]
 
           ## if 'trend' then augment the xs-invariant component
-          if(trend) tHhat <- cbind(tHhat, 1:(dim(tHhat)[[1]]))
+          if(trend) tHhat <- cbind(tHhat, 1:(dim(tHhat)[[1L]]))
 
           ## NB tHat, tMhat should be i-invariant
           tMhat <- diag(1, length(ty)) -
               tHhat %*% solve(crossprod(tHhat), t(tHhat))
-          tXMX <- crossprod(tX, tMhat %*% tX)
-          tXMy <- crossprod(tX, tMhat %*% ty)
+          
+          CP.tXtMhat <- crossprod(tX, tMhat)
+          tXMX <- tcrossprod(CP.tXtMhat, t(tX))
+          tXMy <- tcrossprod(CP.tXtMhat, t(ty))
 
           ## XMX_i, XMy_i
           XMX[ , , i] <- tXMX
@@ -225,10 +224,11 @@ pcce <- function (formula, data, subset, na.action,
           tcoef[ , i] <- tb
 
           ## cce (defactored) residuals as M_i(y_i - X_i * bCCEMG_i)
-          cceres[[i]] <- tMhat %*% (ty - tX %*% tb)
+          tytXtb <- ty - tcrossprod(tX, t(tb))
+          cceres[[i]] <- tcrossprod(tMhat, t(tytXtb))
           ## std. (raw) residuals as y_i - X_i * bCCEMG_i - a_i
           ta <- mean(ty - tX)
-          stdres[[i]] <- ty - tX %*% tb - ta
+          stdres[[i]] <- tytXtb - ta
         }
 
   ## module for making transformed data My, MX for vcovHC use
@@ -236,12 +236,12 @@ pcce <- function (formula, data, subset, na.action,
     ## Some redundancy because this might be moved to model.matrix.pcce
 
     ## initialize
-    tX1 <- X[ind == unind[1], , drop = FALSE]
-    ty1 <- y[ind == unind[1]]
-    tHhat1 <- Hhat[ind == unind[1], , drop = FALSE]
+    tX1 <- X[ind == unind[1L], , drop = FALSE]
+    ty1 <- y[ind == unind[1L]]
+    tHhat1 <- Hhat[ind == unind[1L], , drop = FALSE]
 
     ## if 'trend' then augment the xs-invariant component
-    if(trend) tHhat1 <- cbind(tHhat1, 1:(dim(tHhat)[[1]]))
+    if(trend) tHhat1 <- cbind(tHhat1, 1:(dim(tHhat)[[1L]]))
 
     ## NB tHat, tMhat should be i-invariant (but beware of unbalanced)
     tMhat1 <- diag(1, length(ty1)) -
@@ -254,7 +254,7 @@ pcce <- function (formula, data, subset, na.action,
         tHhat <- Hhat[ind == unind[i], , drop = FALSE]
 
         ## if 'trend' then augment the xs-invariant component
-        if(trend) tHhat <- cbind(tHhat, 1:(dim(tHhat)[[1]]))
+        if(trend) tHhat <- cbind(tHhat, 1:(dim(tHhat)[[1L]]))
 
         ## NB tHat, tMhat should be i-invariant
         tMhat <- diag(1, length(ty)) -
@@ -282,34 +282,34 @@ pcce <- function (formula, data, subset, na.action,
     coefmg <- rowMeans(tcoef) # was: apply(tcoef, 1, mean)
 
     ## make matrix of cross-products of demeaned individual coefficients
-    Rmat <- array(dim = c(k, k, n))
+    Rmat <- array(data = NA_real_, dim = c(k, k, n))
 
     ## make b_i - b_CCEMG
     demcoef <- tcoef - coefmg # coefmg gets recycled n times by column
 
     ## calc. coef and vcov according to model
     switch(model,
-           mg = {
+        "mg" = {
             ## assign beta CCEMG
             coef <- coefmg
-            for(i in 1:n) Rmat[,,i] <-  outer(demcoef[,i], demcoef[,i])
-            vcov <- 1/(n*(n-1)) * apply(Rmat, 1:2, sum)
-            },
+            for(i in 1:n) Rmat[ , , i] <- outer(demcoef[ , i], demcoef[ , i])
+            vcov <- 1/(n*(n-1)) * rowSums(Rmat, dims = 2L) # == 1/(n*(n-1)) * apply(Rmat, 1:2, sum), but rowSums(., dims = 2L)-construct is way faster
+        },
            
-           p = {
+        "p" = {
             ## calc beta_CCEP
-            sXMX <- apply(XMX, 1:2, sum)
-            sXMy <- apply(XMy, 1:2, sum)
+            sXMX <- rowSums(XMX, dims = 2L) # == apply(XMX, 1:2, sum), but rowSums(., dims = 2L)-construct is way faster
+            sXMy <- rowSums(XMy, dims = 2L) # == apply(XMy, 1:2, sum), but rowSums(., dims = 2L)-construct is way faster
             coef <- solve(sXMX, sXMy)
     
             ## calc CCEP covariance:
             psi.star <- 1/N * sXMX
     
-            for(i in 1:n) Rmat[,,i] <- XMX[,,i] %*%
-                outer(demcoef[,i], demcoef[,i]) %*% XMX[,,i]
+            for(i in 1:n) Rmat[ , , i] <- XMX[ , , i] %*%
+                outer(demcoef[ , i], demcoef[ , i]) %*% XMX[ , , i]
             ## summing over the n-dimension of the array we get the
             ## covariance matrix of coefs
-            R.star <- 1/(n-1) * apply(Rmat, 1:2, sum) * 1/(t^2)
+            R.star <- 1/(n-1) * rowSums(Rmat, dims = 2L) * 1/(t^2) # rowSums(Rmat, dims = 2L) faster than == apply(Rmat, 1:2, sum)
     
             Sigmap.star <- solve(psi.star, R.star) %*% solve(psi.star)
             vcov <- Sigmap.star/n
@@ -323,7 +323,7 @@ pcce <- function (formula, data, subset, na.action,
                 tHhat <- Hhat[ind == unind[i], , drop = FALSE]
     
                 ## if 'trend' then augment the xs-invariant component
-                if(trend) tHhat <- cbind(tHhat, 1:(dim(tHhat)[[1]]))
+                if(trend) tHhat <- cbind(tHhat, 1:(dim(tHhat)[[1L]]))
     
                 ## NB tHat, tMhat should be i-invariant (but for the
                 ## group size if unbalanced)
@@ -331,16 +331,17 @@ pcce <- function (formula, data, subset, na.action,
                     tHhat %*% solve(crossprod(tHhat), t(tHhat))
     
                 ## cce residuals as M_i(y_i - X_i * bCCEP)
-                cceres[[i]] <- tMhat %*% (ty - tX %*% coef)
+                tytXcoef <- ty - tcrossprod(tX, t(coef))
+                cceres[[i]] <- tcrossprod(tMhat, t(tytXcoef))
                 ## std. (raw) residuals as y_i - X_i * bCCEMG_i - a_i
-                ta <- mean(ty-tX)
-                stdres[[i]] <- ty - tX %*% coef - ta
+                ta <- mean(ty - tX)
+                stdres[[i]] <- tytXcoef - ta
             }
     })
 
     ## calc. measures of fit according to model type
     switch(model,
-           mg = {
+        "mg" = {
 
             ## R2 as in HPY 2010: sigma2ccemg = average (over n) of variances
             ## of defactored residuals
@@ -358,15 +359,14 @@ pcce <- function (formula, data, subset, na.action,
                 sigma2cce.i[[i]] <- crossprod(cceres[[i]])*
                     1/(length(cceres[[i]])-2*k-2)
             }
-            sigma2cce <- 1/n*sum(unlist(sigma2cce.i))
-            },
+            sigma2cce <- 1/n*sum(unlist(sigma2cce.i, use.names = FALSE))
+        },
            
-           p = {
-
+        "p" = {
             ## variance of defactored residuals sigma2ccep as in Holly,
             ## Pesaran and Yamagata, (3.15)
             sigma2cce <- 1/(n*(T.-k-2)-k)*
-                sum(unlist(lapply(cceres, crossprod)))
+                sum(vapply(cceres, crossprod, FUN.VALUE = 0.0, USE.NAMES = FALSE))
             ## is the same as sum(unlist(cceres)^2)
     })
 
@@ -374,9 +374,9 @@ pcce <- function (formula, data, subset, na.action,
     sigma2.i <- vector("list", n)
     for(i in 1:n) {
           ty <- y[ind == unind[i]]
-          sigma2.i[[i]] <- sum((ty-mean(ty))^2)/(length(ty)-1)
+          sigma2.i[[i]] <- as.numeric(crossprod((ty-mean(ty))))/(length(ty)-1)
       }
-    sigma2y <- mean(unlist(sigma2.i))
+    sigma2y <- mean(unlist(sigma2.i, use.names = FALSE))
     r2cce <- 1 - sigma2cce/sigma2y
 
     ## allow outputting different types of residuals
@@ -386,7 +386,7 @@ pcce <- function (formula, data, subset, na.action,
     ## add transformed data (for now a simple list)
     tr.model <- list(y = My, X = MX)
     ## so that if the model is ccepmod,
-    ## > lm(ccepmod$tr.model[["y"]]~ccepmod$tr.model[["X"]]-1)
+    ## > lm(ccepmod$tr.model[["y"]] ~ ccepmod$tr.model[["X"]]-1)
     ## reproduces the model results
 
     ## Final model object:
@@ -401,15 +401,20 @@ pcce <- function (formula, data, subset, na.action,
     dimnames(tcoef) <- list(coef.names, id.names)
     pmodel <- attr(plm.model, "pmodel")
     pmodel$model.name <- model.name
-    pccemod <- list(coefficients = coef, residuals = residuals,
-                  stdres = stdres, tr.model = tr.model,
-                  fitted.values = fitted.values, vcov = vcov,
-                  df.residual = df.residual,
-                  model = model.frame(plm.model), sigma = NULL,
-                  indcoef = tcoef, r.squared = r2cce,
-                  #cceres = as.vector(cceres),
-                  #ccemgres = as.vector(ccemgres),
-                  formula = formula, call = cl)
+    pccemod <- list(coefficients  = coef,
+                    residuals     = residuals,
+                    stdres        = stdres,
+                    tr.model      = tr.model,
+                    fitted.values = fitted.values,
+                    vcov          = vcov,
+                    df.residual   = df.residual,
+                    model         = model.frame(plm.model),
+                    indcoef       = tcoef,
+                    r.squared     = r2cce,
+                    #cceres   = as.vector(cceres),
+                    #ccemgres = as.vector(ccemgres),
+                    formula       = formula,
+                    call          = cl)
     pccemod <- structure(pccemod, pdim = pdim, pmodel = pmodel)
     class(pccemod) <- c("pcce", "panelmodel")
     pccemod
@@ -433,7 +438,7 @@ summary.pcce <- function(object, vcov = NULL, ...){
   CoefTable <- cbind(b, std.err, z, p)
   colnames(CoefTable) <- c("Estimate", "Std. Error", "z-value", "Pr(>|z|)")
   object$CoefTable <- CoefTable
-  y <- object$model[[1]]
+  y <- object$model[[1L]]
   object$tss <- tss(y)
   object$ssr <- as.numeric(crossprod(residuals(object)))
   object$rsqr <- object$r.squared #1-object$ssr/object$tss
@@ -465,7 +470,7 @@ print.summary.pcce <- function(x, digits = max(3, getOption("digits") - 2), widt
   cat("\n")
   print(pdim)
   cat("\nResiduals:\n")
-  print(summary(unlist(residuals(x))))
+  print(sumres(x)) # was until rev. 1178: print(summary(unlist(residuals(x))))
   cat("\nCoefficients:\n")
   printCoefmat(x$CoefTable, digits = digits)
   cat(paste("Total Sum of Squares: ",    signif(x$tss,digits), "\n", sep=""))
@@ -483,12 +488,12 @@ residuals.pcce <- function(object,
     ## defactored residuals (default) or raw residuals
     defres <- pres(object)
     switch(match.arg(type),
-           standard = {
+           "standard" = {
                ## add panel features and names from 'defres'
                residuals <- add_pseries_features(object$stdres, index(defres))
                names(residuals) <- names(defres)
               },
-           defactored = { residuals <- defres }
+           "defactored" = { residuals <- defres }
            )
     return(residuals)
 }
